from typing import List, Union

import numpy as np
import soundfile as sf

from transcribe import Transcribe

SampleRate = 16000  # Stream device recording frequency per second
BlockSize = 30  # Block size in milliseconds
Vocals = [50, 1000]  # Frequency range to detect sounds that could be speech
EndBlocks = 33 * 2  # Number of blocks to wait before sending (30 ms is block)
FlushBlocks = 33 * 10  # Number of blocks to wait before sending

try:
    import sounddevice as sd

    sounddevice_available = True
    sounddevice_exception = None

except Exception as e:
    sounddevice_available = False
    sounddevice_exception = e


class Live:
    def __init__(
        self,
        model_path: str,
        cache_directory: str,
        local_files_only: bool,
        task: str,
        language: str,
        threads: int,
        device: str,
        device_index: Union[int, List[int]],
        compute_type: str,
        verbose: bool,
        threshold: float,
        input_device: int,
        #        options: TranscriptionOptions,
    ):
        self.model_path = model_path
        self.cache_directory = cache_directory
        self.local_files_only = local_files_only
        self.task = task
        self.language = language
        self.threads = threads
        self.device = device
        self.device_index = device_index
        self.compute_type = compute_type
        self.verbose = verbose
        self.threshold = threshold
        self.input_device = input_device
        #       self.options = options

        self.running = True
        self.waiting = 0
        self.prevblock = self.buffer = np.zeros((0, 1))
        self.speaking = False
        self.blocks_speaking = 0
        self.buffers_to_process = []
        self.transcribe = None

    @staticmethod
    def is_available():
        return sounddevice_available

    @staticmethod
    def force_not_available_exception():
        raise (sounddevice_exception)

    def _is_there_voice(self, indata, frames):
        freq = np.argmax(np.abs(np.fft.rfft(indata[:, 0]))) * SampleRate / frames
        volume = np.sqrt(np.mean(indata**2))

        return volume > self.threshold and Vocals[0] <= freq <= Vocals[1]

    def _is_there_voice2(self, indata, frames):
        # Apply a window function to the signal for better frequency analysis
        window = np.hanning(len(indata))
        indata = indata * window[:, None]

        freq = np.argmax(np.abs(np.fft.rfft(indata[:, 0]))) * SampleRate / frames
        volume = np.sqrt(np.mean(indata**2))

        return volume > self.threshold and Vocals[0] <= freq <= Vocals[1]

    def _save_to_process(self):
        self.buffers_to_process.append(self.buffer.copy())
        self.buffer = np.zeros((0, 1))
        self.speaking = False

    def callback(self, indata, frames, _time, status):
        if not indata.any():
            return

        voice = self._is_there_voice(indata, frames)

        # Silence and no nobody has started speaking
        if not voice and not self.speaking:
            return

        if voice:  # User speaking
            if self.verbose:
                print(".", end="", flush=True)
            if self.waiting < 1:
                self.buffer = self.prevblock.copy()

            self.buffer = np.concatenate((self.buffer, indata))
            self.waiting = EndBlocks

            if not self.speaking:
                self.blocks_speaking = FlushBlocks

            self.speaking = True
        else:  # Silence after user has spoken
            self.waiting -= 1
            if self.waiting < 1:
                self._save_to_process()
                return
            else:
                self.buffer = np.concatenate((self.buffer, indata))

        self.blocks_speaking -= 1
        # User spoken for a long time and we need to flush
        if self.blocks_speaking < 1:
            self._save_to_process()

    def process(self):
        if len(self.buffers_to_process) > 0:
            _buffer = self.buffers_to_process.pop(0)
            if self.verbose:
                print("\n\033[90mTranscribing..\033[0m")

            if not self.transcribe:
                self.transcribe = Transcribe(
                    self.model_path,
                    self.device,
                    self.device_index,
                    self.compute_type,
                    self.threads,
                    self.cache_directory,
                    self.local_files_only,
                )

            result = self.transcribe.inference(
                audio=_buffer.flatten().astype("float32"),
                task=self.task,
                language=self.language,
                verbose=self.verbose,
                live=True,
                options=self.options,
            )
            print(f"\033[1A\033[2K\033[0G{result['text']}")
            if not self.verbose:
                print("")

    def listen(self):
        show_device = (
            self.input_device if self.input_device is not None else sd.default.device[0]
        )
        print(
            f"\033[32mLive stream device: \033[37m{sd.query_devices(device=show_device)['name']}\033[0m"
        )
        print("\033[32mListening.. \033[37m(Ctrl+C to Quit)\033[0m")
        with sd.InputStream(
            channels=1,
            callback=self.callback,
            blocksize=int(SampleRate * BlockSize / 1000),
            samplerate=SampleRate,
            device=self.input_device,
        ):
            while self.running:
                self.process()

    def inference(self):
        try:
            self.listen()
        except (KeyboardInterrupt, SystemExit):
            pass
        finally:
            print("\n\033[93mQuitting..\033[0m")

    def read_wav_file(self, filepath):
        data, samplerate = sf.read(filepath)
        if samplerate != SampleRate:
            raise ValueError(f"Expected sample rate {SampleRate}, but got {samplerate}")
        return data

    def process_wav_file(self, filepath):
        data = self.read_wav_file(filepath)
        frames_per_block = int(SampleRate * BlockSize / 1000)
        for start in range(0, len(data), frames_per_block):
            block = data[start : start + frames_per_block]
            if len(block) < frames_per_block:
                block = np.pad(
                    block, ((0, frames_per_block - len(block)), (0, 0)), "constant"
                )
            self.callback(block, frames_per_block, None, None)
            self.process()
        self.running = False
        self.process()  # Process remaining buffers

    # Example usage


if __name__ == "__main__":
    #    options = TranscriptionOptions()  # Define your transcription options
    live_transcriber = Live(
        model_path="path_to_model",
        cache_directory="path_to_cache",
        local_files_only=True,
        task="transcribe",
        language="en",
        threads=1,
        device="cpu",
        device_index=0,
        compute_type="float32",
        verbose=True,
        threshold=0.01,
        input_device=None,
        #        options=None,
    )
    live_transcriber.process_wav_file("dosparlants.wav")



import argparse

from .languages import LANGUAGES, TO_LANGUAGE_CODE
from .version import __version__

MODEL_NAMES = [
    "tiny",
    "tiny.en",
    "base",
    "base.en",
    "small",
    "small.en",
    "medium",
    "medium.en",
    "large-v1",
    "large-v2",
    "large-v3",
    "large-v3-turbo",
    "turbo",
    "distil-large-v2",
    "distil-large-v3",
    "distil-medium.en",
    "distil-small.en",
]


class CommandLine:
    @staticmethod
    def _optional_int(string):
        return None if string == "None" else int(string)

    @staticmethod
    def _str2bool(string):
        str2val = {"true": True, "false": False}
        if string and string.lower() in str2val:
            return str2val[string.lower()]
        else:
            raise ValueError(f"Expected one of {set(str2val.keys())}, got {string}")

    @staticmethod
    def _optional_float(string):
        return None if string == "None" else float(string)

    @staticmethod
    def read_command_line():
        parser = argparse.ArgumentParser(
            formatter_class=argparse.ArgumentDefaultsHelpFormatter
        )
        parser.add_argument(
            "audio", nargs="*", type=str, help="audio file(s) to transcribe"
        )

        model_args = parser.add_argument_group("Model selection options")

        model_args.add_argument(
            "--model",
            default="small",
            choices=MODEL_NAMES,
            help="name of the Whisper model to use",
        )

        model_args.add_argument(
            "--model_directory",
            type=str,
            default=None,
            help="directory where to find a CTranslate2 Whisper model (e.g. fine-tuned model)",
        )

        caching_args = parser.add_argument_group("Model caching control options")

        caching_args.add_argument(
            "--model_dir",
            type=str,
            default=None,
            help="the path to save model files; uses ~/.cache/huggingface/ by default",
        )

        caching_args.add_argument(
            "--local_files_only",
            type=CommandLine._str2bool,
            default=False,
            help="use only models in cache without connecting to Internet to check if there are newer versions",
        )

        outputs_args = parser.add_argument_group(
            "Configuration options to control generated outputs"
        )

        outputs_args.add_argument(
            "--output_dir",
            "-o",
            type=str,
            default=".",
            help="directory to save the outputs",
        )
        outputs_args.add_argument(
            "--output_format",
            "-f",
            type=str,
            default="all",
            choices=["txt", "vtt", "srt", "tsv", "json", "all"],
            help="format of the output file; if not specified, all available formats will be produced",
        )

        outputs_args.add_argument(
            "--pretty_json",
            "-p",
            type=CommandLine._str2bool,
            default=False,
            help="produce json in a human readable format",
        )

        outputs_args.add_argument(
            "--print_colors",
            type=CommandLine._str2bool,
            default=False,
            help="print the transcribed text using an experimental color coding strategy to highlight words with high or low confidence",
        )

        outputs_args.add_argument(
            "--verbose",
            type=CommandLine._str2bool,
            default=True,
            help="whether to print out the progress and debug messages",
        )

        outputs_args.add_argument(
            "--highlight_words",
            type=CommandLine._str2bool,
            default=False,
            help="underline each word as it is spoken in srt and vtt output formats (requires --word_timestamps True)",
        )

        outputs_args.add_argument(
            "--max_line_width",
            type=CommandLine._optional_int,
            default=None,
            help="the maximum number of characters in a line before breaking the line in srt and vtt output formats (requires --word_timestamps True)",
        )
        outputs_args.add_argument(
            "--max_line_count",
            type=CommandLine._optional_int,
            default=None,
            help="the maximum number of lines in a segment in srt and vtt output formats (requires --word_timestamps True)",
        )
        outputs_args.add_argument(
            "--max_words_per_line",
            type=CommandLine._optional_int,
            default=None,
            help="(requires --word_timestamps True, no effect with --max_line_width) the maximum number of words in a segment",
        )
        computing_args = parser.add_argument_group("Computing configuration options")

        computing_args.add_argument(
            "--device",
            choices=[
                "auto",
                "cpu",
                "cuda",
            ],
            default="auto",
            help="device to use for CTranslate2 inference",
        )

        computing_args.add_argument(
            "--threads",
            type=CommandLine._optional_int,
            default=0,
            help="number of threads used for CPU inference",
        )

        computing_args.add_argument(
            "--device_index",
            type=int,
            default=0,
            help="device ID where to place this model on",
        )

        computing_args.add_argument(
            "--compute_type",
            choices=[
                "default",
                "auto",
                "int8",
                "int8_float16",
                "int8_bfloat16",
                "int8_float32",
                "int16",
                "float16",
                "float32",
                "bfloat16",
            ],
            default="auto",
            help="Type of quantization to use (see https://opennmt.net/CTranslate2/quantization.html)",
        )

        algorithm_args = parser.add_argument_group("Algorithm execution options")

        algorithm_args.add_argument(
            "--task",
            type=str,
            default="transcribe",
            choices=["transcribe", "translate"],
            help="whether to perform X->X speech recognition ('transcribe') or X->English translation ('translate')",
        )
        algorithm_args.add_argument(
            "--language",
            type=str,
            default=None,
            choices=sorted(LANGUAGES.keys())
            + sorted([k.title() for k in TO_LANGUAGE_CODE.keys()]),
            help="language spoken in the audio, specify None to perform language detection",
        )

        algorithm_args.add_argument(
            "--temperature",
            type=float,
            default=0,
            help="temperature to use for sampling",
        )

        algorithm_args.add_argument(
            "--temperature_increment_on_fallback",
            type=CommandLine._optional_float,
            default=0.2,
            help="temperature to increase when falling back when the decoding fails to meet either of the thresholds below",
        )

        algorithm_args.add_argument(
            "--prompt_reset_on_temperature",
            type=float,
            default=0.5,
            help="resets prompt if temperature is above this value. Arg has effect only if condition_on_previous_text is True",
        )

        algorithm_args.add_argument(
            "--prefix",
            type=str,
            default=None,
            help="optional text to provide as a prefix for the first window",
        )

        algorithm_args.add_argument(
            "--best_of",
            type=CommandLine._optional_int,
            default=5,
            help="number of candidates when sampling with non-zero temperature",
        )
        algorithm_args.add_argument(
            "--beam_size",
            type=CommandLine._optional_int,
            default=5,
            help="number of beams in beam search, only applicable when temperature is zero",
        )
        algorithm_args.add_argument(
            "--patience",
            type=float,
            default=1.0,
            help="optional patience value to use in beam decoding, as in https://arxiv.org/abs/2204.05424, the default (1.0) is equivalent to conventional beam search",
        )
        algorithm_args.add_argument(
            "--length_penalty",
            type=float,
            default=1.0,
            help="optional token length penalty coefficient (alpha) as in https://arxiv.org/abs/1609.08144, uses simple length normalization by default",
        )
        algorithm_args.add_argument(
            "--suppress_blank",
            type=CommandLine._str2bool,
            default=True,
            help="suppress blank outputs at the beginning of the sampling",
        )
        algorithm_args.add_argument(
            "--suppress_tokens",
            type=str,
            default="-1",
            help="comma-separated list of token ids to suppress during sampling; '-1' will suppress most special characters except common punctuations",
        )
        algorithm_args.add_argument(
            "--initial_prompt",
            type=str,
            default=None,
            help="optional text to provide as a prompt for the first window.",
        )
        algorithm_args.add_argument(
            "--condition_on_previous_text",
            type=CommandLine._str2bool,
            default=True,
            help="if True, provide the previous output of the model as a prompt for the next window; disabling may make the text inconsistent across windows, but the model becomes less prone to getting stuck in a failure loop",
        )
        algorithm_args.add_argument(
            "--compression_ratio_threshold",
            type=CommandLine._optional_float,
            default=2.4,
            help="if the gzip compression ratio is higher than this value, treat the decoding as failed",
        )
        algorithm_args.add_argument(
            "--logprob_threshold",
            type=CommandLine._optional_float,
            default=-1.0,
            help="if the average log probability is lower than this value, treat the decoding as failed",
        )
        algorithm_args.add_argument(
            "--no_speech_threshold",
            type=CommandLine._optional_float,
            default=0.6,
            help="if the probability of the <|nospeech|> token is higher than this value AND the decoding has failed due to `logprob_threshold`, consider the segment as silence",
        )
        algorithm_args.add_argument(
            "--word_timestamps",
            type=CommandLine._str2bool,
            default=False,
            help="(experimental) extract word-level timestamps and refine the results based on them",
        )

        algorithm_args.add_argument(
            "--prepend_punctuations",
            type=str,
            default="\"'“¿([{-",
            help="if word_timestamps is True, merge these punctuation symbols with the next word",
        )
        algorithm_args.add_argument(
            "--append_punctuations",
            type=str,
            default="\"'.。,，!！?？:：”)]}、",
            help="if word_timestamps is True, merge these punctuation symbols with the previous word",
        )
        algorithm_args.add_argument(
            "--repetition_penalty",
            type=float,
            default=1.0,
            help="penalty applied to the score of previously generated tokens (set > 1 to penalize)",
        )
        algorithm_args.add_argument(
            "--no_repeat_ngram_size",
            type=int,
            default=0,
            help="prevent repetitions of ngrams with this size (set 0 to disable)",
        )
        algorithm_args.add_argument(
            "--hallucination_silence_threshold",
            type=CommandLine._optional_float,
            default=None,
            help="When word_timestamps is True, skip silent periods longer than this threshold (in seconds) when a possible hallucination is detected",
        )
        algorithm_args.add_argument(
            "--hotwords",
            type=str,
            default=None,
            help="Hotwords/hint phrases to the model. Useful for names you want the model to priotize",
        )

        algorithm_args.add_argument(
            "--batched",
            type=CommandLine._str2bool,
            default=False,
            help="Uses Batched transcription which can provide an additional 2x-3x speed increase",
        )

        vad_args = parser.add_argument_group("VAD filter arguments")

        vad_args.add_argument(
            "--vad_filter",
            type=CommandLine._str2bool,
            default=False,
            help="enable the voice activity detection (VAD) to filter out parts of the audio without speech. This step is using the Silero VAD model https://github.com/snakers4/silero-vad.",
        )

        vad_args.add_argument(
            "--vad_onset",
            type=float,
            default=None,
            help="when `vad_filter` is enabled, probabilities above this value are considered as speech. This parameter was called `vad_threshold` before",
        )

        vad_args.add_argument(
            "--vad_min_speech_duration_ms",
            type=int,
            default=None,
            help="when `vad_filter` is enabled, final speech chunks shorter min_speech_duration_ms are thrown out.",
        )

        vad_args.add_argument(
            "--vad_max_speech_duration_s",
            type=int,
            default=None,
            help="when `vad_filter` is enabled, Maximum duration of speech chunks in seconds. Longer will be split at the timestamp of the last silence.",
        )

        vad_args.add_argument(
            "--vad_min_silence_duration_ms",
            type=int,
            default=None,
            help="when `vad_filter` is enabled, in the end of each speech chunk time to wait before separating it.",
        )

        parser.add_argument(
            "--version",
            action="version",
            version="%(prog)s {version}".format(version=__version__),
            help="show program's version number and exit",
        )

        diarization_args = parser.add_argument_group("Diarization options")
        diarization_args.add_argument(
            "--hf_token",
            type=str,
            default="",
            help="HuggingFace token which enables to download the diarization models.",
        )

        diarization_args.add_argument(
            "--speaker_name",
            type=str,
            default="SPEAKER",
            help="Name to use to identify the speaker (e.g. SPEAKER_00).",
        )

        live_args = parser.add_argument_group("Live transcribe options")

        live_args.add_argument(
            "--live_transcribe",
            type=CommandLine._str2bool,
            default=False,
            help="live transcribe mode",
        )

        live_args.add_argument(
            "--live_volume_threshold",
            type=float,
            default=0.2,
            help="minimum volume threshold to activate listening in live transcribe mode",
        )

        live_args.add_argument(
            "--live_input_device",
            type=int,
            default=None,
            help="Set live stream input device ID (see python -m sounddevice for a list)",
        )

        return parser.parse_args().__dict__



#
# Based on code from https://github.com/openai/whisper
#

import json
import os
import re

from typing import Callable, Optional, TextIO


def format_timestamp(
    seconds: float, always_include_hours: bool = False, decimal_marker: str = "."
):
    assert seconds >= 0, "non-negative timestamp expected"
    milliseconds = round(seconds * 1000.0)

    hours = milliseconds // 3_600_000
    milliseconds -= hours * 3_600_000

    minutes = milliseconds // 60_000
    milliseconds -= minutes * 60_000

    seconds = milliseconds // 1_000
    milliseconds -= seconds * 1_000

    hours_marker = f"{hours:02d}:" if always_include_hours or hours > 0 else ""
    return (
        f"{hours_marker}{minutes:02d}:{seconds:02d}{decimal_marker}{milliseconds:03d}"
    )


class ResultWriter:
    extension: str

    def __init__(self, output_dir: str):
        self.output_dir = output_dir

    def __call__(self, result: dict, audio_path: str, options: dict):
        audio_basename = os.path.basename(audio_path)
        audio_basename = os.path.splitext(audio_basename)[0]
        output_path = os.path.join(
            self.output_dir, audio_basename + "." + self.extension
        )

        with open(output_path, "w", encoding="utf-8") as f:
            self.write_result(result, f, options)

    def write_result(self, result: dict, file: TextIO, options: dict):
        raise NotImplementedError


class SubtitlesWriter(ResultWriter):
    always_include_hours: bool
    decimal_marker: str

    def iterate_result(self, result: dict, options: dict):
        raw_max_line_width: Optional[int] = options.get("max_line_width", None)
        raw_max_words_per_line: Optional[int] = options.get("max_words_per_line", None)
        max_line_count: Optional[int] = options.get("max_line_count", None)
        highlight_words = options.get("highlight_words", False)
        max_line_width = 1000 if raw_max_line_width is None else raw_max_line_width
        max_words_per_line = (
            1000 if raw_max_words_per_line is None else raw_max_words_per_line
        )
        preserve_segments = (
            max_line_count is None
            or raw_max_line_width is None
            or raw_max_words_per_line is None
        )

        def iterate_subtitles():
            line_len = 0
            line_count = 1
            # the next subtitle to yield (a list of word timings with whitespace)
            subtitle: list[dict] = []
            last = result["segments"][0]["words"][0]["start"]
            for segment in result["segments"]:
                speaker = f"[{segment['speaker']}]: " if "speaker" in segment else ""
                chunk_index = 0
                words_count = max_words_per_line
                while chunk_index < len(segment["words"]):
                    remaining_words = len(segment["words"]) - chunk_index
                    if max_words_per_line > len(segment["words"]) - chunk_index:
                        words_count = remaining_words
                    for i, original_timing in enumerate(
                        segment["words"][chunk_index : chunk_index + words_count]
                    ):
                        timing = original_timing.copy()
                        long_pause = (
                            not preserve_segments and timing["start"] - last > 3.0
                        )
                        has_room = line_len + len(timing["word"]) <= max_line_width
                        seg_break = i == 0 and len(subtitle) > 0 and preserve_segments
                        if (
                            line_len > 0
                            and has_room
                            and not long_pause
                            and not seg_break
                        ):
                            # line continuation
                            line_len += len(timing["word"])
                        else:
                            # new line
                            timing["word"] = speaker + timing["word"].strip()
                            speaker = ""
                            if (
                                len(subtitle) > 0
                                and max_line_count is not None
                                and (long_pause or line_count >= max_line_count)
                                or seg_break
                            ):
                                # subtitle break
                                yield subtitle
                                subtitle = []
                                line_count = 1
                            elif line_len > 0:
                                # line break
                                line_count += 1
                                timing["word"] = "\n" + timing["word"]
                            line_len = len(timing["word"].strip())
                        subtitle.append(timing)
                        last = timing["start"]
                    chunk_index += max_words_per_line
            if len(subtitle) > 0:
                yield subtitle

        if (
            len(result["segments"]) > 0
            and "words" in result["segments"][0]
            and result["segments"][0]["words"] is not None
            and len(result["segments"][0]["words"]) > 0
        ):
            for subtitle in iterate_subtitles():
                subtitle_start = self.format_timestamp(subtitle[0]["start"])
                subtitle_end = self.format_timestamp(subtitle[-1]["end"])
                subtitle_text = "".join([word["word"] for word in subtitle])
                if highlight_words:
                    last = subtitle_start
                    all_words = [timing["word"] for timing in subtitle]
                    for i, this_word in enumerate(subtitle):
                        start = self.format_timestamp(this_word["start"])
                        end = self.format_timestamp(this_word["end"])
                        if last != start:
                            yield last, start, subtitle_text

                        yield start, end, "".join(
                            [
                                (
                                    re.sub(r"^(\s*)(.*)$", r"\1<u>\2</u>", word)
                                    if j == i
                                    else word
                                )
                                for j, word in enumerate(all_words)
                            ]
                        )
                        last = end
                else:
                    yield subtitle_start, subtitle_end, subtitle_text
        else:
            for segment in result["segments"]:
                speaker = f"[{segment['speaker']}]: " if "speaker" in segment else ""
                segment_start = self.format_timestamp(segment["start"])
                segment_end = self.format_timestamp(segment["end"])
                segment_text = speaker + segment["text"].strip().replace("-->", "->")
                yield segment_start, segment_end, segment_text

    def format_timestamp(self, seconds: float):
        return format_timestamp(
            seconds=seconds,
            always_include_hours=self.always_include_hours,
            decimal_marker=self.decimal_marker,
        )


class WriteTXT(ResultWriter):
    extension: str = "txt"

    def write_result(self, result: dict, file: TextIO, options: dict):
        for segment in result["segments"]:
            speaker = f"[{segment['speaker']}]: " if "speaker" in segment else ""
            print(speaker + segment["text"].strip(), file=file, flush=True)


class WriteSRT(SubtitlesWriter):
    extension: str = "srt"
    always_include_hours: bool = True
    decimal_marker: str = ","

    def write_result(self, result: dict, file: TextIO, options: dict):
        for i, (start, end, text) in enumerate(
            self.iterate_result(result, options), start=1
        ):
            print(f"{i}\n{start} --> {end}\n{text}\n", file=file, flush=True)


class WriteVTT(SubtitlesWriter):
    extension: str = "vtt"
    always_include_hours: bool = False
    decimal_marker: str = "."

    def write_result(self, result: dict, file: TextIO, options: dict):
        print("WEBVTT\n", file=file)
        for start, end, text in self.iterate_result(result, options):
            print(f"{start} --> {end}\n{text}\n", file=file, flush=True)


class WriteTSV(ResultWriter):
    """
    Write a transcript to a file in TSV (tab-separated values) format containing lines like:
    <start time in integer milliseconds>\t<end time in integer milliseconds>\t<transcript text>

    Using integer milliseconds as start and end times means there's no chance of interference from
    an environment setting a language encoding that causes the decimal in a floating point number
    to appear as a comma; also is faster and more efficient to parse & store, e.g., in C++.
    """

    extension: str = "tsv"

    def write_result(self, result: dict, file: TextIO, options: dict):
        print("start", "end", "text", sep="\t", file=file)
        for segment in result["segments"]:
            print(round(1000 * segment["start"]), file=file, end="\t")
            print(round(1000 * segment["end"]), file=file, end="\t")
            print(segment["text"].strip().replace("\t", " "), file=file, flush=True)


class WriteJSON(ResultWriter):
    extension: str = "json"

    def write_result(self, result: dict, file: TextIO, options: dict):
        pretty_json: bool = options.get("pretty_json", False)

        if pretty_json:
            json.dump(result, file, indent=4, ensure_ascii=False)
        else:
            json.dump(result, file)


def get_writer(
    output_format: str, output_dir: str
) -> Callable[[dict, TextIO, dict], None]:
    writers = {
        "txt": WriteTXT,
        "vtt": WriteVTT,
        "srt": WriteSRT,
        "tsv": WriteTSV,
        "json": WriteJSON,
    }

    if output_format == "all":
        all_writers = [writer(output_dir) for writer in writers.values()]

        def write_all(result: dict, file: TextIO, options: dict):
            for writer in all_writers:
                writer(result, file, options)

        return write_all

    return writers[output_format](output_dir)



import unittest

from src.whisper_ctranslate2.live import Live


class TestLive(unittest.TestCase):
    def test_constructor(self):
        live = Live(
            model_path="path",
            cache_directory="directory",
            local_files_only=False,
            task="translate",
            language="ca",
            threads="10",
            device="auto",
            device_index=0,
            compute_type="int8",
            verbose=False,
            threshold=0.2,
            input_device=0,
            options=None,
        )
        self.assertNotEqual(None, live)


if __name__ == "__main__":
    unittest.main()
